@@ -85,6 +85,7 @@ from optparse import OptionParser
 
 from scalyr_agent.profiler import ScalyrProfiler
 from scalyr_agent.scalyr_client import ScalyrClientSession
+from scalyr_agent.scalyr_client import create_client, verify_server_certificate
 from scalyr_agent.copying_manager import CopyingManager
 from scalyr_agent.configuration import Configuration
 from scalyr_agent.util import RunState, ScriptEscalator
@@ -100,6 +101,7 @@ from scalyr_agent.platform_controller import (
 )
 from scalyr_agent.platform_controller import AgentNotRunning
 from scalyr_agent.build_info import get_build_revision
+from scalyr_agent import compat
 
 
 STATUS_FILE = "last_status"
@@ -111,6 +113,26 @@ AGENT_LOG_FILENAME = "agent.log"
 
 AGENT_NOT_RUNNING_MESSAGE = "The agent does not appear to be running."
 
+# Message which is logged when locale used for the scalyr agent process is not UTF-8
+NON_UTF8_LOCALE_WARNING_MESSAGE = """
+Detected a non UTF-8 locale (%s) being used. You are strongly encouraged to set the locale /
+coding for the agent process to UTF-8. Otherwise things won't work when trying to monitor files
+with non-ascii content or non-ascii characters in the log file names. On Linux you can do that by
+setting LANG and LC_ALL environment variable: e.g. export LC_ALL=en_US.UTF-8.
+""".strip().replace(
+    "\n", " "
+)
+
+# Work around for a potential race which may happen when threads try to resolve a unicode hostname
+# or similar
+# See:
+# - https://bugs.python.org/issue29288
+# - https://github.com/aws/aws-cli/pull/4383/files#diff-45cb40c448cb3c90162a08a8d5c86559afb843a7678339500c3fb15933b5dcceR55
+try:
+    "".encode("idna")
+except Exception as e:
+    print("Failed to force idna encoding: %s" % (str(e)))
+
 
 def _update_disabled_until(config_value, current_time):
     if config_value is not None:
@@ -171,10 +193,6 @@ class ScalyrAgent(object):
         self.__copying_manager = None
         # The current monitors manager.
         self.__monitors_manager = None
-        # The current ScalyrClientSession to use for sending requests.
-        self.__scalyr_client = None
-        # The current NewScalyrClientSession to use for sending requests.
-        self.__new_scalyr_client = None
 
         # Tracks whether or not the agent should still be running.  When a terminate signal is received,
         # the run state is set to false.  Threads are expected to notice this and finish as quickly as
@@ -523,7 +541,7 @@ class ScalyrAgent(object):
 
         # Send a test message to the server to make sure everything works.  If not, print a decent error message.
         if not no_check_remote or self.__config.use_new_ingestion:
-            client = self.__create_client(quiet=True)
+            client = create_client(self.__config, quiet=True)
             try:
                 ping_result = client.ping()
                 if ping_result != "success":
@@ -1029,27 +1047,16 @@ class ScalyrAgent(object):
                 else:
                     logs_initial_positions = None
 
-                # 2->TODO it was very helpful to see what python version does agent run on. Maybe we can keep it?
-                python_version_str = sys.version.replace("\n", "")
-                build_revision = get_build_revision()
-                openssl_version = getattr(ssl, "OPENSSL_VERSION", "unknown")
+                start_up_msg = scalyr_util.get_agent_start_up_message()
+                log.info(start_up_msg)
+                log.log(scalyr_logging.DEBUG_LEVEL_1, start_up_msg)
 
-                # TODO: Why do we log the same line under info and debug? Intentional?
-                msg = (
-                    "Starting scalyr agent... (version=%s) (revision=%s) %s (Python version: %s) "
-                    "(OpenSSL version: %s) (default fs encoding: %s)"
-                    % (
-                        SCALYR_VERSION,
-                        build_revision,
-                        scalyr_util.get_pid_tid(),
-                        python_version_str,
-                        openssl_version,
-                        sys.getfilesystemencoding(),
-                    )
-                )
-
-                log.info(msg)
-                log.log(scalyr_logging.DEBUG_LEVEL_1, msg)
+                # Log warn message if non UTF-8 locale is used - this would cause issues when trying
+                # to monitor files with unicode characters inside the file names or inside the
+                # content
+                _, encoding, _ = scalyr_util.get_language_code_coding_and_locale()
+                if encoding.lower() not in ["utf-8", "utf8"]:
+                    log.warn(NON_UTF8_LOCALE_WARNING_MESSAGE % (encoding))
 
                 self.__controller.emit_init_log(log, self.__config.debug_init)
 
@@ -1065,20 +1072,16 @@ class ScalyrAgent(object):
                 config_pre_global_apply = self.__config
                 self.__config.print_useful_settings()
 
-                self.__scalyr_client = self.__create_client()
-                self.__new_scalyr_client = self.__create_new_client()
+                # verify server certificates.
+                verify_server_certificate(self.__config)
 
                 def start_worker_thread(config, logs_initial_positions=None):
                     wt = self.__create_worker_thread(config)
                     # attach callbacks before starting monitors
                     wt.monitors_manager.set_user_agent_augment_callback(
-                        self.__scalyr_client.augment_user_agent
-                    )
-                    wt.start(
-                        self.__scalyr_client,
-                        self.__new_scalyr_client,
-                        logs_initial_positions,
+                        wt.copying_manager.augment_user_agent_for_workers_sessions
                     )
+                    wt.start(logs_initial_positions)
                     return wt, wt.copying_manager, wt.monitors_manager
 
                 (
@@ -1350,8 +1353,6 @@ class ScalyrAgent(object):
 
                     prev_server = scalyr_server
 
-                    self.__scalyr_client = self.__create_client()
-
                     log.info("Starting new copying and metrics threads")
                     (
                         worker_thread,
@@ -1447,65 +1448,6 @@ class ScalyrAgent(object):
             result = NewScalyrClientSession(self.__config)
         return result
 
-    def __create_client(self, quiet=False):
-        """Creates and returns a new client to the Scalyr servers.
-
-        @param quiet: If true, only errors should be written to stdout.
-        @type quiet: bool
-
-        @return: The client to use for sending requests to Scalyr, using the server address and API write logs
-            key in the configuration file.
-        @rtype: ScalyrClientSession
-        """
-        if (
-            self.__config.verify_server_certificate
-            and not self.__config.use_new_ingestion
-        ):
-            is_dev_install = INSTALL_TYPE == DEV_INSTALL
-            is_dev_or_msi_install = INSTALL_TYPE in [DEV_INSTALL, MSI_INSTALL]
-
-            ca_file = self.__config.ca_cert_path
-            intermediate_certs_file = self.__config.intermediate_certs_path
-
-            # Validate provided CA cert file and intermediate cert file exists. If they don't
-            # exist, throw and fail early and loudly
-            if not is_dev_install and not os.path.isfile(ca_file):
-                raise ValueError(
-                    'Invalid path "%s" specified for the "ca_cert_path" config '
-                    "option: file does not exist" % (ca_file)
-                )
-
-            # NOTE: We don't include intermediate certs in the Windows binary so we skip that check
-            # under the MSI / Windows install
-            if not is_dev_or_msi_install and not os.path.isfile(
-                intermediate_certs_file
-            ):
-                raise ValueError(
-                    'Invalid path "%s" specified for the '
-                    '"intermediate_certs_path" config '
-                    "option: file does not exist" % (intermediate_certs_file)
-                )
-        else:
-            ca_file = None
-            intermediate_certs_file = None
-        use_requests_lib = self.__config.use_requests_lib
-        return ScalyrClientSession(
-            self.__config.scalyr_server,
-            self.__config.api_key,
-            SCALYR_VERSION,
-            quiet=quiet,
-            request_deadline=self.__config.request_deadline,
-            ca_file=ca_file,
-            intermediate_certs_file=intermediate_certs_file,
-            use_requests_lib=use_requests_lib,
-            compression_type=self.__config.compression_type,
-            compression_level=self.__config.compression_level,
-            proxies=self.__config.network_proxies,
-            disable_send_requests=self.__config.disable_send_requests,
-            disable_logfile_addevents_format=self.__config.disable_logfile_addevents_format,
-            enforce_monotonic_timestamps=self.__config.enforce_monotonic_timestamps,
-        )
-
     def __get_file_initial_position(self, path):
         """Returns the file size for the specified file.
 
@@ -1729,10 +1671,10 @@ class ScalyrAgent(object):
         stats = overall_stats
 
         log.info(
-            "copy_manager_status total_copy_iterations=%ld total_read_time=%lf total_compression_time=%lf total_waiting_time=%lf total_blocking_response_time=%lf "
+            "copy_manager_status total_scan_iterations=%ld total_read_time=%lf total_compression_time=%lf total_waiting_time=%lf total_blocking_response_time=%lf "
             "total_request_time=%lf total_pipelined_requests=%ld avg_bytes_produced_rate=%lf avg_bytes_copied_rate=%lf"
             % (
-                stats.total_copy_iterations,
+                stats.total_scan_iterations,
                 stats.total_read_time,
                 stats.total_compression_time,
                 stats.total_waiting_time,
@@ -1750,7 +1692,7 @@ class ScalyrAgent(object):
         """Return a newly calculated overall stats for the agent.
 
         This will calculate the latest stats based on the running agent.  Since most stats only can be
-        calculated since the last time the configuration file changed and was read, we need to seperately
+        calculated since the last time the configuration file changed and was read, we need to separately
         track the accumulated stats that occurred before the last config change.
 
         @param base_overall_stats: The accummulated stats from before the last config change.
@@ -1768,7 +1710,7 @@ class ScalyrAgent(object):
         watched_paths = 0
         copying_paths = 0
 
-        # Accumulate all the stats from the running processors that are copying log files.
+        # Accumulate all the stats from the running processors in all workers that are copying log files.
         if current_status.copying_manager_status is not None:
             delta_stats.total_copy_requests_errors = (
                 current_status.copying_manager_status.total_errors
@@ -1776,9 +1718,6 @@ class ScalyrAgent(object):
             delta_stats.total_rate_limited_time = (
                 current_status.copying_manager_status.total_rate_limited_time
             )
-            delta_stats.total_copy_iterations = (
-                current_status.copying_manager_status.total_copy_iterations
-            )
             delta_stats.total_read_time = (
                 current_status.copying_manager_status.total_read_time
             )
@@ -1797,6 +1736,11 @@ class ScalyrAgent(object):
             delta_stats.rate_limited_time_since_last_status = (
                 current_status.copying_manager_status.rate_limited_time_since_last_status
             )
+
+            delta_stats.total_scan_iterations = (
+                current_status.copying_manager_status.total_scan_iterations
+            )
+
             watched_paths = len(current_status.copying_manager_status.log_matchers)
             for matcher in current_status.copying_manager_status.log_matchers:
                 copying_paths += len(matcher.log_processors_status)
@@ -1839,24 +1783,28 @@ class ScalyrAgent(object):
                 )
                 delta_stats.total_monitor_errors += monitor_status.errors
 
-        delta_stats.total_requests_sent = self.__scalyr_client.total_requests_sent
-        delta_stats.total_requests_failed = self.__scalyr_client.total_requests_failed
-        delta_stats.total_request_bytes_sent = (
-            self.__scalyr_client.total_request_bytes_sent
-        )
-        delta_stats.total_compressed_request_bytes_sent = (
-            self.__scalyr_client.total_compressed_request_bytes_sent
-        )
-        delta_stats.total_response_bytes_received = (
-            self.__scalyr_client.total_response_bytes_received
-        )
-        delta_stats.total_request_latency_secs = (
-            self.__scalyr_client.total_request_latency_secs
-        )
-        delta_stats.total_connections_created = (
-            self.__scalyr_client.total_connections_created
-        )
-        delta_stats.total_compression_time = self.__scalyr_client.total_compression_time
+        # get client session states from all workers of the copying manager and sum up all their stats.
+        session_states = self.__copying_manager.get_worker_session_statuses()
+        for session_state in session_states:
+
+            delta_stats.total_requests_sent += session_state.total_requests_sent
+            delta_stats.total_requests_failed += session_state.total_requests_failed
+            delta_stats.total_request_bytes_sent += (
+                session_state.total_request_bytes_sent
+            )
+            delta_stats.total_compressed_request_bytes_sent += (
+                session_state.total_compressed_request_bytes_sent
+            )
+            delta_stats.total_response_bytes_received += (
+                session_state.total_response_bytes_received
+            )
+            delta_stats.total_request_latency_secs += (
+                session_state.total_request_latency_secs
+            )
+            delta_stats.total_connections_created += (
+                session_state.total_connections_created
+            )
+            delta_stats.total_compression_time += session_state.total_compression_time
 
         # Add in the latest stats to the stats before the last restart.
         result = delta_stats + base_overall_stats
@@ -2013,21 +1961,13 @@ class WorkerThread(object):
     """
 
     def __init__(self, configuration, copying_manager, monitors):
-        self.__scalyr_client = None
-        self.__new_scalyr_client = None
         self.config = configuration
         self.copying_manager = copying_manager
         self.monitors_manager = monitors
 
-    def start(self, scalyr_client, new_scalyr_client, log_initial_positions=None):
-        if self.__scalyr_client is not None:
-            self.__scalyr_client.close()
-        self.__scalyr_client = scalyr_client
-        self.__new_scalyr_client = new_scalyr_client
+    def start(self, log_initial_positions=None):
 
-        self.copying_manager.start_manager(
-            scalyr_client, new_scalyr_client, log_initial_positions
-        )
+        self.copying_manager.start_manager(log_initial_positions)
         # We purposely wait for the copying manager to begin copying so that if the monitors create any new
         # files, they will be guaranteed to be copying up to the server starting at byte index zero.
         # Note, if copying never begins then the copying manager will sys exit, so this next call will never just
@@ -2042,10 +1982,6 @@ class WorkerThread(object):
         log.debug("Shutting copy monitors")
         self.copying_manager.stop_manager()
 
-        log.debug("Shutting client")
-        if self.__scalyr_client is not None:
-            self.__scalyr_client.close()
-
 
 if __name__ == "__main__":
     my_controller = PlatformController.new_platform()
@@ -2112,7 +2048,7 @@ if __name__ == "__main__":
         dest="no_check_remote",
         help="For the start command, does not perform the first check to see if the agent can "
         "communicate with the Scalyr servers.  The agent will just keep trying to contact it in "
-        "the backgroudn until it is successful.  This is useful if the network is not immediately "
+        "the background until it is successful.  This is useful if the network is not immediately "
         "available when the agent starts.",
     )
     my_controller.add_options(parser)
