@@ -45,6 +45,7 @@ from optparse import OptionParser
 if not sys.platform.startswith("win"):
     from pwd import getpwnam
 
+# pylint: disable=import-error
 from __scalyr__ import (
     scalyr_init,
     get_install_root,
@@ -54,6 +55,8 @@ from __scalyr__ import (
     PACKAGE_INSTALL,
 )
 
+# pylint: enable=import-error
+
 scalyr_init()
 
 # [start of 2->TODO]
@@ -1505,7 +1508,9 @@ if __name__ == "__main__":
         sys.exit(1)
 
     try:
-        config_file = Configuration(options.config_filename, default_paths, log)
+        config_file = Configuration(
+            options.config_filename, default_paths, log, log_warnings=False
+        )
         config_file.parse()
     except Exception as e:
         print(
