@@ -19,17 +19,26 @@
 #
 # author: Steven Czerwinski <czerwin@scalyr.com>
 
-__author__ = 'czerwin@scalyr.com'
+from __future__ import absolute_import
+from scalyr_agent import compat
 
-import cStringIO
+__author__ = "czerwin@scalyr.com"
+
+import io
 import errno
 import socket
-import SocketServer
+import six.moves.socketserver
 import struct
 import time
 
+import scalyr_agent.scalyr_logging as scalyr_logging
 
-class ServerProcessor(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
+global_log = scalyr_logging.getLogger(__name__)
+
+
+class ServerProcessor(
+    six.moves.socketserver.ThreadingMixIn, six.moves.socketserver.TCPServer
+):
     """Base class for simple servers that only need to accept incoming connections, perform some actions on
     individual commands, and return no output.
 
@@ -42,8 +51,16 @@ class ServerProcessor(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
     methods.  Additionally, you should probably provide an implementation of report_connection_problem so
     that errors generated will handling an individual connection can be recorded.
     """
-    def __init__(self, server_port, localhost_socket=True, max_connection_idle_time=300, max_request_size=100*1024,
-                 buffer_size=100*1024, run_state=None):
+
+    def __init__(
+        self,
+        server_port,
+        localhost_socket=True,
+        max_connection_idle_time=300,
+        max_request_size=100 * 1024,
+        buffer_size=100 * 1024,
+        run_state=None,
+    ):
         """Creates a new instance that will accept new connections on the specified port.
 
         To begin accepting connections, you must invoke the 'run' method.
@@ -60,9 +77,9 @@ class ServerProcessor(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
             connections.
         """
         if localhost_socket:
-            server_address = ('localhost', server_port)
+            server_address = ("localhost", server_port)
         else:
-            server_address = ('', server_port)
+            server_address = ("", server_port)
         # Allow for quick server socket re-use.  This helps prevent problems if we are quickly restarting the agent.
         self.allow_reuse_address = True
         self.max_connection_idle_time = max_connection_idle_time
@@ -73,7 +90,9 @@ class ServerProcessor(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
         if run_state is not None:
             self.run_state.register_on_stop_callback(self.shutdown)
 
-        SocketServer.TCPServer.__init__(self, server_address, ConnectionHandler)
+        six.moves.socketserver.TCPServer.__init__(
+            self, server_address, ConnectionHandler
+        )
 
     def run(self):
         """Begins accepting new connections and processing the incoming requests.
@@ -134,12 +153,15 @@ class LineRequestParser(object):
     """Simple abstraction that implements a 'parse_request' that can be used to parse incoming requests
     that are terminated by either '\n' or '\r\n'.
     """
-    def __init__(self, max_request_size):
+
+    def __init__(self, max_request_size, eof_as_eol=False):
         """Creates a new instance.
 
         @param max_request_size: The maximum number of bytes that can be contained in an individual request.
+        @param eof_as_eol: If true then treat EOF as the end of a line
         """
         self.__max_request_size = max_request_size
+        self.__eof_as_eol = eof_as_eol
 
     def parse_request(self, input_buffer, _):
         """Returns the next complete request from 'input_buffer'.
@@ -169,7 +191,21 @@ class LineRequestParser(object):
 
             # Check to see if a complete line was actually returned since if readline() hit EOF, then
             # it will return whatever line was left without a newline.
-            if bytes_received > 0 and line[-1] == '\n':
+            return_line = False
+            if bytes_received > 0:
+                # 2->TODO use slicing to get bytes on bith python versions.
+                if line[-1:] == b"\n":
+                    return_line = True
+                else:
+                    # check if we want to return the remaining text when EOF is hit
+                    if self.__eof_as_eol:
+                        # this will return an empty string if EOF is reached
+                        last_line = input_buffer.read(1)
+                        if len(last_line) == 0:
+                            # we have reach eof so also return
+                            return_line = True
+
+            if return_line:
                 original_position = None
                 return line
         finally:
@@ -186,6 +222,7 @@ class Int32RequestParser(object):
 
     Some monitor protocols, such as Graphite's pickle protocol uses this format.
     """
+
     def __init__(self, max_request_size):
         """Creates a new instance.
 
@@ -217,7 +254,11 @@ class Int32RequestParser(object):
             # Make sure we have 4 bytes so that we can at least read the length prefix, and then try to read
             # the complete data payload.
             if num_bytes > self.__prefix_length:
-                length, = struct.unpack(self.__format, input_buffer.read(self.__prefix_length))
+                # 2->TODO struct.pack|unpack in python < 2.7.7 does not allow unicode format string.
+                (length,) = compat.struct_unpack_unicode(
+                    six.ensure_str(self.__format),
+                    input_buffer.read(self.__prefix_length),
+                )
                 if length > self.__max_request_size:
                     raise RequestSizeExceeded(length, self.__max_request_size)
                 if length + self.__prefix_length <= num_bytes:
@@ -234,17 +275,25 @@ class ConnectionIdleTooLong(Exception):
     """Raised when the time since a connection last received a complete request has exceeded the maximum connection
     idle time.
     """
+
     def __init__(self, time_since_last_activity, max_inactivity):
-        Exception.__init__(self, 'Connection has been idle too long. No data has been received for '
-                                 '%d seconds and limit is %d' % (time_since_last_activity, max_inactivity))
+        Exception.__init__(
+            self,
+            "Connection has been idle too long. No data has been received for "
+            "%d seconds and limit is %d" % (time_since_last_activity, max_inactivity),
+        )
 
 
 class RequestSizeExceeded(Exception):
     """Raised when an incoming request has exceeded the maximum allowed request size.
     """
+
     def __init__(self, request_size, max_request_size):
-        Exception.__init__(self, 'The current request size of %d exceeded maximum allowed of %d' % (
-                           request_size, max_request_size))
+        Exception.__init__(
+            self,
+            "The current request size of %d exceeded maximum allowed of %d"
+            % (request_size, max_request_size),
+        )
 
 
 class ConnectionProcessor(object):
@@ -254,11 +303,14 @@ class ConnectionProcessor(object):
     complete request has not been received in the allowed time.
 
     """
+
     # This abstraction exists really only for testing purposes.  It could have been implemented as part of
     # 'ConnectionHandler', but since that class derives from 'SocketServer.BaseRequestHandler', it is
     # difficult to test.  That is because 'SocketServer.BaseRequestHandler' does non-test-friendly things like
     # invoking 'handle' as part of instance initialization.
-    def __init__(self, request_stream, request_executor, run_state, max_connection_idle_time):
+    def __init__(
+        self, request_stream, request_executor, run_state, max_connection_idle_time
+    ):
         """Returns a new instance.  You must invoke 'run' to begin processing requests.
 
         @param request_stream: An instance of 'RequestSream' containing the incoming bytes for a connection.
@@ -299,31 +351,43 @@ class ConnectionProcessor(object):
         if self.__last_request_time is None:
             self.__last_request_time = current_time
 
-        request = self.__request_stream.read_request(timeout=self.__polling_interval, run_state=self.__run_state)
+        request = self.__request_stream.read_request(
+            timeout=self.__polling_interval, run_state=self.__run_state
+        )
         if request is not None:
             self.__request_executor(request)
             self.__last_request_time = current_time
         elif current_time - self.__last_request_time > self.__max_connection_idle_time:
-            raise ConnectionIdleTooLong(self.__last_request_time - time.time(), self.__max_connection_idle_time)
+            raise ConnectionIdleTooLong(
+                self.__last_request_time - time.time(), self.__max_connection_idle_time
+            )
         return self.__run_state.is_running() and not self.__request_stream.at_end()
 
 
-class ConnectionHandler(SocketServer.BaseRequestHandler):
+class ConnectionHandler(six.moves.socketserver.BaseRequestHandler):
     """The handler class that is used by ServerProcess to handle incoming connections.
     """
+
     # The bulk of the work for this class is actually implemented in ConnectionProcess to allow for
     # easier testing.
     def handle(self):
         try:
             # Create an instance of RequestStream for the incoming connection and then use a ConnectionProcessor
             # to handle it.
-            request_stream = RequestStream(self.request, self.server.parse_request,
-                                           max_buffer_size=self.server.buffer_size,
-                                           max_request_size=self.server.max_request_size)
-            processor = ConnectionProcessor(request_stream, self.server.execute_request, self.server.run_state,
-                                            self.server.max_connection_idle_time)
+            request_stream = RequestStream(
+                self.request,
+                self.server.parse_request,
+                max_buffer_size=self.server.buffer_size,
+                max_request_size=self.server.max_request_size,
+            )
+            processor = ConnectionProcessor(
+                request_stream,
+                self.server.execute_request,
+                self.server.run_state,
+                self.server.max_connection_idle_time,
+            )
             processor.run()
-        except Exception, e:
+        except Exception as e:
             self.server.report_connection_problem(e)
 
 
@@ -333,7 +397,15 @@ class RequestStream(object):
     This essentially puts a memory buffer in front of an incoming socket to efficiently read requests from
     the incoming stream and reset the read position when required by incomplete requests.
     """
-    def __init__(self, incoming_socket, request_parser, max_buffer_size=100*1024, max_request_size=100*1024):
+
+    def __init__(
+        self,
+        incoming_socket,
+        request_parser,
+        max_buffer_size=100 * 1024,
+        max_request_size=100 * 1024,
+        blocking=True,
+    ):
         """Creates a new instance.
 
         @param incoming_socket: The incoming socket.
@@ -342,6 +414,8 @@ class RequestStream(object):
             specifying the number of bytes available in the buffer.
         @param max_buffer_size: The maximum buffer to use for buffering the incoming requests.
         @param max_request_size: The maximum allowed size for each request.
+        @param blocking: If True pause before reading from the socket to allow time for data to come int
+            if False read directly from the socket.
         """
         # We use non-blocking sockets so that we can response quicker when the agent is
         # shutting down.  It does mean there will be some delay between bytes showing up on the
@@ -351,18 +425,26 @@ class RequestStream(object):
         self.__request_parser = request_parser
         self.__max_buffer_size = max_buffer_size
         self.__max_request_size = max_request_size
+        self.__blocking = blocking
 
         if self.__max_buffer_size < self.__max_request_size:
-            raise Exception('You cannot have a max buffer size smaller than your max request size (%d > %d)' % (
-                            self.__max_buffer_size, self.__max_request_size))
+            raise Exception(
+                "You cannot have a max buffer size smaller than your max request size (%d > %d)"
+                % (self.__max_buffer_size, self.__max_request_size)
+            )
 
         # The number of bytes in _buffer.
         self.__current_buffer_size = 0
         # Whether or not the socket has been closed and the stream should be considered at the end.
         self.__at_end = False
+
+        # Whether or not a socket error has occurred - we keep this separate from at_end to be able to
+        # distinguish between errors and eof.  This is useful when testing.
+        self.__socket_error = False
+
         # The actual buffer.  We will maintain an invariant that the position of the buffer is always pointing at
         # the next byte to read.
-        self.__buffer = cStringIO.StringIO()
+        self.__buffer = io.BytesIO()
 
     def read_request(self, timeout=0.5, run_state=None):
         """Attempts to read the next complete request from the socket and return it.
@@ -383,22 +465,50 @@ class RequestStream(object):
         try:
             try:
                 # Try to read the request from the already existing buffered input if possible.
-                bytes_available_to_read = self.__get_buffer_write_position() - self.__get_buffer_read_position()
+                bytes_available_to_read = (
+                    self.__get_buffer_write_position()
+                    - self.__get_buffer_read_position()
+                )
                 if bytes_available_to_read > 0:
-                    parsed_request = self.__request_parser(self.__buffer, bytes_available_to_read)
+                    parsed_request = self.__request_parser(
+                        self.__buffer, bytes_available_to_read
+                    )
                     if parsed_request is not None:
                         do_full_compaction = False
                         return parsed_request
+                    elif bytes_available_to_read >= self.__max_request_size:
+                        # The parser didn't return a request even though the maximum request size has been reached..
+                        # This should never happen (if parser is written correctly), so throw an error
+                        raise RequestSizeExceeded(
+                            bytes_available_to_read, bytes_available_to_read
+                        )
+                    elif self.__max_buffer_size == self.__get_buffer_write_position():
+                        # If there are pending bytes left in the buffer, then they didn't form a full request.  We
+                        # definitely need to read more bytes from the network, so do a full compaction if we have to.
+                        self.__full_compaction()
 
-                # No data immediately available.  Wait a few milliseconds for some more to come in.
-                if not self.__sleep_until_timeout_or_stopped(timeout, run_state):
-                    return None
+                if self.__blocking:
+                    # No data immediately available.  Wait a few milliseconds for some more to come in.
+                    if not self.__sleep_until_timeout_or_stopped(timeout, run_state):
+                        return None
 
                 do_full_compaction = True
-                data = self.__socket.recv(self.__max_buffer_size - self.__get_buffer_write_position())
+
+                if self.__max_buffer_size - self.__get_buffer_write_position() == 0:
+                    global_log.warning(
+                        "RequestStream: write_position == max_buffer.  No room in recv buffer"
+                    )
+
+                data = self.__socket.recv(
+                    self.__max_buffer_size - self.__get_buffer_write_position()
+                )
                 # If we get nothing back, then the connection has been closed.  If it is not closed and there is
                 # no data, then we would get a socket.timeout or socket.error which are handled below.
                 if not data:
+                    global_log.log(
+                        scalyr_logging.DEBUG_LEVEL_1,
+                        "RequestStream.read_request: No data received, flagging end of stream",
+                    )
                     self.__at_end = True
                     return None
 
@@ -407,34 +517,45 @@ class RequestStream(object):
                 self.__add_to_buffer(data)
 
                 # Attempt to parse.
-                parsed_request = self.__request_parser(self.__buffer, bytes_available_to_read)
+                parsed_request = self.__request_parser(
+                    self.__buffer, bytes_available_to_read
+                )
 
                 if parsed_request is not None:
                     # The parser should be checking the max request size as well, but we do a quick double
                     # check here as well.
                     if len(parsed_request) > self.__max_request_size:
-                        raise RequestSizeExceeded(len(parsed_request), self.__max_request_size)
+                        raise RequestSizeExceeded(
+                            len(parsed_request), self.__max_request_size
+                        )
                     do_full_compaction = False
                     return parsed_request
 
                 # If we didn't find a complete request but we are at our maximum buffer size, then we are screwed.. we
                 # don't any more room to try to read more bytes to complete that request.. so just call it an error.
                 if bytes_available_to_read == self.__max_buffer_size:
-                    raise RequestSizeExceeded(self.__max_buffer_size, self.__max_buffer_size)
+                    raise RequestSizeExceeded(
+                        self.__max_buffer_size, self.__max_buffer_size
+                    )
 
                 return None
 
             except socket.timeout:
+                self.__socket_error = True
                 return None
-            except socket.error, e:
+            except socket.error as e:
                 if e.errno == errno.EAGAIN:
                     return None
                 else:
+                    self.__socket_error = True
                     raise e
         finally:
             # We do a full compaction in general if we did not return anything and there is no room
             # left to copy new bytes in.
-            if do_full_compaction and self.__get_buffer_write_position() == self.__max_buffer_size:
+            if (
+                do_full_compaction
+                and self.__get_buffer_write_position() == self.__max_buffer_size
+            ):
                 # Do a compaction if our buffer is at the limit, but we also have bytes at the front of it that have
                 # already been consumed (i.e., are read position is not at the beginning of the buffer.. so we can
                 # reclaiming the bytes before it to make room.)
@@ -448,6 +569,13 @@ class RequestStream(object):
         """
         return self.__at_end
 
+    def is_closed(self):
+        """Returns True if the underlying socket has been closed - either due to reading EOF
+        or due to socket.timeouts and other exceptions.
+        We keep this separate from at_end so callers can detect just for EOF if they want.
+        """
+        return self.at_end() or self.__socket_error
+
     def get_buffer_size(self):
         """Returns the size of the underlying buffer, which may also include bytes for requests already returned
         by 'read_request'.
@@ -508,7 +636,7 @@ class RequestStream(object):
         # Read the leftover data and write it into a new buffer.
         remaining_data = self.__buffer.read()
         self.__buffer.close()
-        self.__buffer = cStringIO.StringIO()
+        self.__buffer = io.BytesIO()
         self.__buffer.write(remaining_data)
         self.__current_buffer_size = self.__buffer.tell()
         self.__buffer.seek(0)
@@ -519,5 +647,5 @@ class RequestStream(object):
         """
         if self.__get_buffer_read_position() == self.__get_buffer_write_position():
             self.__buffer.close()
-            self.__buffer = cStringIO.StringIO()
-            self.__current_buffer_size = 0
\ No newline at end of file
+            self.__buffer = io.BytesIO()
+            self.__current_buffer_size = 0
